@@ -1,5 +1,6 @@
 # These functions are
-# Copyright (C) 1998-2010 T.W. Yee, University of Auckland. All rights reserved.
+# Copyright (C) 1998-2011 T.W. Yee, University of Auckland.
+# All rights reserved.
 
 
 
@@ -94,7 +95,8 @@ bilogistic4.control <- function(save.weight=TRUE, ...)
         Scale2 = eta2theta(eta[,4], .lscale)
         zedd1 = (y[,1]-loc1) / Scale1
         zedd2 = (y[,2]-loc2) / Scale2
-        if (residuals) stop("loglikelihood residuals not implemented yet") else
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else
         sum(w * (-zedd1 - zedd2 - 3 * log1p(exp(-zedd1)+exp(-zedd2)) -
                  log(Scale1) - log(Scale2)))
     }, list(.lscale=lscale, .llocation=llocation))),
@@ -260,7 +262,8 @@ dbilogis4 = function(x1, x2, loc1=0, scale1=1, loc2=0, scale2=1, log=FALSE) {
         alphap = eta2theta(eta[,2], .lap)
         beta   = eta2theta(eta[,3], .lb)
         betap  = eta2theta(eta[,4], .lbp)
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             tmp88 = extra$y1.lt.y2
             ell1 = log(alpha[tmp88]) + log(betap[tmp88]) -
                    betap[tmp88] * y[tmp88,2] -
@@ -309,109 +312,158 @@ dbilogis4 = function(x1, x2, loc1=0, scale1=1, loc2=0, scale2=1, log=FALSE) {
 
 
 
- mckaygamma2 = function(la="loge",
-                        lp="loge",
-                        lq="loge",
-                        ia=NULL,
-                        ip=1,
-                        iq=1,
-                        zero=NULL) {
-    if (mode(la) != "character" && mode(la) != "name")
-        la = as.character(substitute(la))
-    if (mode(lp) != "character" && mode(lp) != "name")
-        lp = as.character(substitute(lp))
-    if (mode(lq) != "character" && mode(lq) != "name")
-        lq = as.character(substitute(lq))
-    if (!is.Numeric(ip, positive = TRUE) || !is.Numeric(iq, positive = TRUE))
-        stop("initial values for 'ip' and 'iq' must be positive")
-    if (is.Numeric(ia) && any(ia <= 0))
-        stop("'ia' must be positive or NULL")
+
+
+
+ bivgamma.mckay = function(lscale="loge",
+                        lshape1="loge",
+                        lshape2="loge",
+                        iscale=NULL,
+                        ishape1=NULL,
+                        ishape2=NULL,
+                        method.init=1,
+                        zero=1) {
+    if (mode(lscale) != "character" && mode(lscale) != "name")
+        lscale = as.character(substitute(lscale))
+    if (mode(lshape1) != "character" && mode(lshape1) != "name")
+        lshape1 = as.character(substitute(lshape1))
+    if (mode(lshape2) != "character" && mode(lshape2) != "name")
+        lshape2 = as.character(substitute(lshape2))
+    if (!is.null(iscale))
+        if (!is.Numeric(iscale, positive = TRUE))
+            stop("'iscale' must be positive or NULL")
+    if (!is.null(ishape1))
+        if (!is.Numeric(ishape1, positive = TRUE))
+            stop("'ishape1' must be positive or NULL")
+    if (!is.null(ishape2))
+        if (!is.Numeric(ishape2, positive = TRUE))
+            stop("'ishape2' must be positive or NULL")
+    if (!is.Numeric(method.init, allow=1, integ=TRUE, positi=TRUE) ||
+       method.init > 2.5)
+        stop("argument 'method.init' must be 1 or 2")
 
     new("vglmff",
-    blurb=c("McKay's Bivariate Gamma Distribution\n",
+    blurb=c("Bivariate Gamma: McKay's Distribution\n",
            "Links:    ",
-           namesof("a", la), ", ",
-           namesof("p", lp), ", ",
-           namesof("q", lq)),
+           namesof("scale", lscale), ", ",
+           namesof("shape1", lshape1), ", ",
+           namesof("shape2", lshape2)),
     constraints=eval(substitute(expression({
         constraints = cm.zero.vgam(constraints, x, .zero, M)
-    }), list(.zero=zero))),
+    }), list( .zero=zero ))),
     initialize=eval(substitute(expression({
         if (!is.matrix(y) || ncol(y) != 2)
-            stop("the response must be a 2 column matrix") 
-        sorty1 = pmin(y[,1], y[,2])
-        sorty2 = pmax(y[,1], y[,2])
-        if (any(sorty2 - sorty1 <= 0))
-            stop("Delete those observations that are identical")
-        predictors.names = c(namesof("a", .la, short=TRUE), 
-                      namesof("p", .lp, short=TRUE), 
-                      namesof("q", .lq, short=TRUE))
+            stop("the response must be a 2 column matrix")
+        if (any(y[,1] >= y[,2]))
+            stop("the second column minus the first column must be a vector ",
+                  "of positive values")
+        predictors.names = c(namesof("scale", .lscale, short=TRUE), 
+                             namesof("shape1", .lshape1, short=TRUE), 
+                             namesof("shape2", .lshape2, short=TRUE))
         if (!length(etastart)) {
-            pinit = rep(.ip, len=n)
-            qinit = rep(.iq, len=n)
-            ainit = if (length(.ia)) {
-                      rep(.ia, len=n) 
-                    } else (pinit+qinit)/(sorty1+0.1)
-            etastart = cbind(theta2eta(ainit, .la),
-                             theta2eta(pinit, .lp),
-                             theta2eta(qinit, .lq))
+            momentsY = if ( .method.init == 1) {
+                cbind(median(y[,1]),  # This may not be monotonic
+                      median(y[,2])) + 0.01
+            } else {
+                cbind(weighted.mean(y[,1], w),
+                      weighted.mean(y[,2], w))
+            }
+
+            mcg2.loglik = function(thetaval, y, x, w, extraargs) {
+                ainit = a = thetaval
+                momentsY = extraargs$momentsY
+                p = (1/a) * abs(momentsY[1]) + 0.01
+                q = (1/a) * abs(momentsY[2] - momentsY[1]) + 0.01
+                sum(w * (-(p+q)*log(a) - lgamma(p) - lgamma(q) +
+                     (p-1)*log(y[,1]) + (q-1)*log(y[,2]-y[,1]) - y[,2] / a ))
+            }
+
+            a.grid = if (length( .iscale )) c( .iscale ) else
+               c(0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1, 2, 5, 10, 20, 50, 100)
+            extraargs = list(momentsY = momentsY)
+            ainit = getMaxMin(a.grid, objfun=mcg2.loglik,
+                              y=y,  x=x, w=w, maximize=TRUE,
+                              extraargs = extraargs)
+            ainit = rep(if(is.Numeric( .iscale )) .iscale else ainit, len=n)
+            pinit = (1/ainit) * abs(momentsY[1]) + 0.01
+            qinit = (1/ainit) * abs(momentsY[2] - momentsY[1]) + 0.01
+
+            pinit = rep(if(is.Numeric( .ishape1 )) .ishape1 else pinit, len=n)
+            qinit = rep(if(is.Numeric( .ishape2 )) .ishape2 else qinit, len=n)
+
+            etastart = cbind(theta2eta(ainit, .lscale),
+                             theta2eta(pinit, .lshape1),
+                             theta2eta(qinit, .lshape2))
         }
-    }), list(.la=la, .lp=lp, .lq=lq, .ia=ia, .ip=ip, .iq=iq))),
+    }), list( .lscale=lscale, .lshape1=lshape1, .lshape2=lshape2,
+              .iscale=iscale, .ishape1=ishape1, .ishape2=ishape2,
+              .method.init=method.init ))),
     inverse=eval(substitute(function(eta, extra=NULL) {
-        a = eta2theta(eta[,1], .la)
-        p = eta2theta(eta[,2], .lp)
-        q = eta2theta(eta[,3], .lq)
-        cbind("pmin(y1,y2)"=(p+q)/a, "pmax(y1,y2)"=NA)
-    }, list(.la=la, .lp=lp, .lq=lq))),
+        a = eta2theta(eta[,1], .lscale)
+        p = eta2theta(eta[,2], .lshape1)
+        q = eta2theta(eta[,3], .lshape2)
+        cbind("y1"=p*a, "y2"=(p+q)*a)
+    }, list( .lscale=lscale, .lshape1=lshape1, .lshape2=lshape2 ))),
     last=eval(substitute(expression({
-        misc$link = c("a"= .la, "p"= .lp, "q"= .lq)
-    }), list(.la=la, .lp=lp, .lq=lq))),
+        misc$link = c("scale"= .lscale, "shape1"= .lshape1, "shape2"= .lshape2)
+        misc$ishape1 = .ishape1
+        misc$ishape2 = .ishape2
+        misc$iscale = .iscale
+        misc$expected = TRUE
+    }), list( .lscale=lscale, .lshape1=lshape1, .lshape2=lshape2,
+              .iscale=iscale, .ishape1=ishape1, .ishape2=ishape2 ))),
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
-        a = eta2theta(eta[,1], .la)
-        p = eta2theta(eta[,2], .lp)
-        q = eta2theta(eta[,3], .lq)
-        y = cbind(pmin(y[,1], y[,2]), pmax(y[,1], y[,2])) # Sort so y[,1]<y[,2]
-        # Note that, after sorting, y[,1] < y[,2] is needed:
-        if (residuals) stop("loglikelihood residuals not implemented yet") else
-        sum(w * ((p+q)*log(a) - lgamma(p) - lgamma(q) +
-                 (p-1)*log(y[,1]) + (q-1)*log(y[,2]-y[,1]) - a*y[,2] ))
-    }, list(.la=la, .lp=lp, .lq=lq))),
-    vfamily=c("mckaygamma2"),
+        a = eta2theta(eta[,1], .lscale)
+        p = eta2theta(eta[,2], .lshape1)
+        q = eta2theta(eta[,3], .lshape2)
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else
+        sum(w * (-(p+q)*log(a) - lgamma(p) - lgamma(q) +
+                  (p-1)*log(y[,1]) + (q-1)*log(y[,2]-y[,1]) - y[,2] / a))
+    }, list( .lscale=lscale, .lshape1=lshape1, .lshape2=lshape2 ))),
+    vfamily=c("bivgamma.mckay"),
     deriv=eval(substitute(expression({
-        a = eta2theta(eta[,1], .la)
-        p = eta2theta(eta[,2], .lp)
-        q = eta2theta(eta[,3], .lq)
-        sorty = y
-        sorty[,1] = pmin(y[,1], y[,2])
-        sorty[,2] = pmax(y[,1], y[,2])
-        d1 = (p+q)/a - sorty[,2]
-        d2 = log(a) - digamma(p) + log(sorty[,1])
-        d3 = log(a) - digamma(q) + log(sorty[,2]-sorty[,1])
-        w * cbind(d1 * dtheta.deta(a, .la),
-                  d2 * dtheta.deta(p, .lp),
-                  d3 * dtheta.deta(q, .lq))
-    }), list(.la=la, .lp=lp, .lq=lq))),
+        aparam = eta2theta(eta[,1], .lscale)
+        shape1 = eta2theta(eta[,2], .lshape1)
+        shape2 = eta2theta(eta[,3], .lshape2)
+        dl.da = (-(shape1+shape2) + y[,2] / aparam) / aparam
+        dl.dshape1 = -log(aparam) - digamma(shape1) + log(y[,1])
+        dl.dshape2 = -log(aparam) - digamma(shape2) + log(y[,2]-y[,1])
+        w * cbind(dl.da      * dtheta.deta(aparam, .lscale),
+                  dl.dshape1 * dtheta.deta(shape1, .lshape1),
+                  dl.dshape2 * dtheta.deta(shape2, .lshape2))
+    }), list( .lscale=lscale, .lshape1=lshape1, .lshape2=lshape2 ))),
     weight=eval(substitute(expression({
-        d11 = (p+q)/a^2
-        d22 = trigamma(p)
-        d33 = trigamma(q)
-        d12 = -1/a
-        d13 = -1/a
+        d11 = (shape1+shape2) / aparam^2
+        d22 = trigamma(shape1)
+        d33 = trigamma(shape2)
+        d12 = 1 / aparam
+        d13 = 1 / aparam
         d23 = 0
         wz = matrix(0, n, dimm(M))
-        wz[,iam(1,1,M)] = dtheta.deta(a, .la)^2 * d11
-        wz[,iam(2,2,M)] = dtheta.deta(p, .lp)^2 * d22
-        wz[,iam(3,3,M)] = dtheta.deta(q, .lq)^2 * d33
-        wz[,iam(1,2,M)] = dtheta.deta(a, .la) * dtheta.deta(p, .lp) * d12
-        wz[,iam(1,3,M)] = dtheta.deta(a, .la) * dtheta.deta(q, .lq) * d13
-        wz[,iam(2,3,M)] = dtheta.deta(p, .lp) * dtheta.deta(q, .lq) * d23
+        wz[,iam(1,1,M)] = dtheta.deta(aparam, .lscale)^2 * d11
+        wz[,iam(2,2,M)] = dtheta.deta(shape1, .lshape1)^2 * d22
+        wz[,iam(3,3,M)] = dtheta.deta(shape2, .lshape2)^2 * d33
+        wz[,iam(1,2,M)] = dtheta.deta(aparam, .lscale) *
+                          dtheta.deta(shape1, .lshape1) * d12
+        wz[,iam(1,3,M)] = dtheta.deta(aparam, .lscale) *
+                          dtheta.deta(shape2, .lshape2) * d13
+        wz[,iam(2,3,M)] = dtheta.deta(shape1, .lshape1) *
+                          dtheta.deta(shape2, .lshape2) * d23
         w * wz
-    }), list(.la=la, .lp=lp, .lq=lq))))
+    }), list( .lscale=lscale, .lshape1=lshape1, .lshape2=lshape2 ))))
 }
 
 
 
+
+
+
+
+
+
+
 rfrank = function(n, alpha) {
     if (!is.Numeric(n, posit=TRUE, allow=1, integ=TRUE)) stop("bad input for 'n'")
     if (!is.Numeric(alpha, posit=TRUE)) stop("bad input for 'alpha'")
@@ -545,7 +597,8 @@ frank.control <- function(save.weight=TRUE, ...)
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         apar = eta2theta(eta, .lapar, earg= .eapar )
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             sum(w * dfrank(x1=y[,1], x2=y[,2], alpha=apar, log = TRUE))
         }
     }, list(.lapar=lapar, .eapar=eapar ))),
@@ -647,7 +700,8 @@ frank.control <- function(save.weight=TRUE, ...)
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         theta = eta2theta(eta, .ltheta)
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             sum(w * (-exp(-theta)*y[,1]/theta - theta*y[,2]))
         }
     }, list(.ltheta=ltheta))),
@@ -730,7 +784,8 @@ frank.control <- function(save.weight=TRUE, ...)
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         alpha  = eta2theta(eta, .lapar, earg= .earg )
         alpha[abs(alpha) < .tola0 ] = .tola0
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
         denom = (1 + alpha - 2*alpha*(exp(-y[,1]) + exp(-y[,2])) +
                 4*alpha*exp(-y[,1] - y[,2]))
         sum(w * (-y[,1] - y[,2] + log(denom)))
@@ -904,7 +959,8 @@ fgm.control <- function(save.weight=TRUE, ...)
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         alpha = eta2theta(eta, .lapar, earg= .earg )
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             sum(w * dfgm(x1=y[,1], x2=y[,2], alpha=alpha, log=TRUE))
         }
     }, list( .lapar=lapar, .earg=earg ))),
@@ -992,7 +1048,8 @@ fgm.control <- function(save.weight=TRUE, ...)
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         alpha  = eta2theta(eta, .lapar, earg= .earg )
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             denom = (alpha*y[,1] - 1) * (alpha*y[,2] - 1) + alpha
             mytolerance = .Machine$double.xmin
             bad <- (denom <= mytolerance)   # Range violation
@@ -1181,7 +1238,8 @@ plackett.control <- function(save.weight=TRUE, ...)
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         oratio = eta2theta(eta, .link, earg= .earg )
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             sum(w * dplack(x1= y[,1], x2= y[,2], oratio=oratio, log=TRUE))
         }
     }, list( .link=link, .earg=earg ))),
@@ -1350,7 +1408,8 @@ amh.control <- function(save.weight=TRUE, ...)
     loglikelihood= eval(substitute(
             function(mu, y, w, residuals = FALSE, eta, extra=NULL) {
         alpha = eta2theta(eta, .lalpha, earg= .ealpha )
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             sum(w * damh(x1=y[,1], x2=y[,2], alpha=alpha, log=TRUE))
         }
     }, list( .lalpha=lalpha, .earg=ealpha ))),
