@@ -1,5 +1,6 @@
 # These functions are
-# Copyright (C) 1998-2010 T.W. Yee, University of Auckland. All rights reserved.
+# Copyright (C) 1998-2011 T.W. Yee, University of Auckland.
+# All rights reserved.
 
 
 
@@ -10,7 +11,7 @@
 
 
  dcnormal1 = function(r1=0, r2=0, link.sd="loge",
-                      earg=list(), 
+                      earg =list(), 
                       isd=NULL, zero=NULL)
 {
     if (!is.Numeric(r1, allow=1, integ=TRUE) || r1<0) stop("bad input for r1")
@@ -22,15 +23,15 @@
     new("vglmff",
     blurb=c("Univariate Normal distribution with double censoring\n\n",
             "Links:    ",
-            "mean; ", namesof("sd", link.sd, earg=earg, tag= TRUE),
+            "mean; ", namesof("sd", link.sd, earg =earg, tag= TRUE),
             "\n",
             "Variance: sd^2"),
     constraints=eval(substitute(expression({
         constraints = cm.zero.vgam(constraints, x, .zero, M)
-    }) , list( .zero=zero))),
+    }) , list( .zero = zero))),
     initialize=eval(substitute(expression({
         predictors.names =
-        c("mean", namesof("sd", .link.sd, earg=.earg, tag= FALSE))
+        c("mean", namesof("sd", .link.sd, earg =.earg, tag= FALSE))
         if (ncol(y <- cbind(y)) != 1)
             stop("the response must be a vector or a one-column matrix")
         if (length(w) != n || !is.Numeric(w, integ=TRUE, posit=TRUE))
@@ -45,10 +46,10 @@
                 1.25 * sqrt( sum(w * junk$resid^2) / junk$df.residual )
             }
             etastart = cbind(mu=y,
-                             theta2eta(sd.y.est, .link.sd, earg=.earg))
+                             theta2eta(sd.y.est, .link.sd, earg =.earg))
         }
     }) , list( .link.sd=link.sd, .r1=r1, .r2=r2, .isd=isd,
-               .earg=earg ))),
+               .earg =earg ))),
     inverse=function(eta, extra=NULL) eta[,1], 
     last=eval(substitute(expression({
         misc$link = c(mu="identity", sd= .link.sd)
@@ -57,19 +58,20 @@
         misc$r1 = .r1
         misc$r2 = .r2
     }) , list( .link.sd=link.sd, .r1=r1, .r2=r2,
-               .earg=earg ))),
+               .earg =earg ))),
     loglikelihood=eval(substitute(
         function(mu,y,w,residuals= FALSE,eta, extra=NULL) {
-        sd = eta2theta(eta[,2], .link.sd, earg=.earg)
-        if (residuals) stop("loglikelihood residuals not implemented yet") else
+        sd = eta2theta(eta[,2], .link.sd, earg =.earg)
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else
         sum(w * (-log(sd) - 0.5 * ((y - mu)/sd)^2)) +
         (if(.r1==0) 0 else {z1=min((y-mu)/sd); Fz1=pnorm(z1); .r1*log(Fz1)}) +
         (if(.r2==0) 0 else {z2=max((y-mu)/sd); Fz2=pnorm(z2); .r2*log1p(-Fz2)})
     } , list( .link.sd=link.sd, .r1=r1, .r2=r2,
-               .earg=earg ))),
+               .earg =earg ))),
     vfamily=c("dcnormal1"),
     deriv=eval(substitute(expression({
-        sd = eta2theta(eta[,2], .link.sd, earg=.earg)
+        sd = eta2theta(eta[,2], .link.sd, earg =.earg)
         q1 = .r1 / extra$bign
         q2 = .r2 / extra$bign
         pee = 1 - q1 - q2  # 1 if r1==r2==0
@@ -84,10 +86,10 @@
         dl.dsd = -1/sd + (y-mu)^2 / sd^3 +
                  ((- .r1 * z1*fz1/Fz1 + .r2 * z2*fz2/(1-Fz2)) / sd) / (n*w)
         dmu.deta = dtheta.deta(mu, "identity") 
-        dsd.deta = dtheta.deta(sd, .link.sd, earg=.earg)
+        dsd.deta = dtheta.deta(sd, .link.sd, earg =.earg)
         cbind(w * dl.dmu * dmu.deta, w * dl.dsd * dsd.deta)
     }) , list( .link.sd=link.sd, .r1=r1, .r2=r2,
-               .earg=earg ))),
+               .earg =earg ))),
     weight=expression({
         wz = matrix(as.numeric(NA), n, dimm(M))
         Q1 = ifelse(q1==0, 1, q1)  # Saves division by 0 below; not elegant
@@ -170,6 +172,11 @@ rbisa = function(n, shape, scale=1) {
 
 
 
+
+
+
+
+
  bisa = function(lshape = "loge", lscale = "loge",
                  eshape = list(), escale = list(),
                  ishape = NULL,   iscale=1,
@@ -192,16 +199,17 @@ rbisa = function(n, shape, scale=1) {
     new("vglmff",
     blurb=c("Birnbaum-Saunders distribution\n\n",
             "Links:    ",
-            namesof("shape", lshape, earg= eshape, tag= TRUE), "; ",
-            namesof("scale", lscale, earg= escale, tag= TRUE)),
+            namesof("shape", lshape, earg = eshape, tag= TRUE), "; ",
+            namesof("scale", lscale, earg = escale, tag= TRUE)),
     constraints=eval(substitute(expression({
         constraints = cm.zero.vgam(constraints, x, .zero, M)
-    }) , list( .zero=zero))),
+    }) , list( .zero = zero))),
     initialize=eval(substitute(expression({
         if (ncol(y <- cbind(y)) != 1)
             stop("the response must be a vector or a one-column matrix")
-        predictors.names = c(namesof("shape", .lshape,earg= .eshape,tag=FALSE),
-                             namesof("scale", .lscale, tag=FALSE))
+        predictors.names =
+          c(namesof("shape", .lshape, earg = .eshape, tag = FALSE),
+            namesof("scale", .lscale, tag=FALSE))
         if (!length(etastart)) {
             scale.init = rep( .iscale, len=n)
             shape.init = if (is.Numeric( .ishape)) rep( .ishape, len=n) else {
@@ -216,55 +224,57 @@ rbisa = function(n, shape, scale=1) {
                     sqrt(2*( pmax(ybar, scale.init+0.1) / scale.init - 1))
                 }
             }
-            etastart = cbind(theta2eta(shape.init, .lshape, earg= .eshape),
-                             theta2eta(scale.init, .lscale, earg= .escale))
+            etastart = cbind(theta2eta(shape.init, .lshape, earg = .eshape),
+                             theta2eta(scale.init, .lscale, earg = .escale))
         }
-    }) , list( .lshape=lshape, .lscale=lscale,
+    }) , list( .lshape = lshape, .lscale = lscale,
                .ishape=ishape, .iscale=iscale,
-               .eshape=eshape, .escale=escale,
+               .eshape = eshape, .escale = escale,
                .method.init=method.init ))),
     inverse=eval(substitute(function(eta, extra=NULL) {
-        sh = eta2theta(eta[,1], .lshape, earg= .eshape)
-        sc = eta2theta(eta[,2], .lscale, earg= .escale)
+        sh = eta2theta(eta[,1], .lshape, earg = .eshape)
+        sc = eta2theta(eta[,2], .lscale, earg = .escale)
         sc * (1 + sh^2 / 2)
-    }, list( .lshape=lshape, .lscale=lscale,
-             .eshape=eshape, .escale=escale ))),
+    }, list( .lshape = lshape, .lscale = lscale,
+             .eshape = eshape, .escale = escale ))),
     last=eval(substitute(expression({
         misc$link = c(shape= .lshape, scale= .lscale)
         misc$earg = list(shape= .eshape, scale= .escale)
         misc$expected = TRUE
-    }) , list( .lshape=lshape, .lscale=lscale,
-               .eshape=eshape, .escale=escale ))),
+    }) , list( .lshape = lshape, .lscale = lscale,
+               .eshape = eshape, .escale = escale ))),
     loglikelihood=eval(substitute(
         function(mu,y,w,residuals= FALSE,eta, extra=NULL) {
-        sh = eta2theta(eta[,1], .lshape, earg= .eshape)
-        sc = eta2theta(eta[,2], .lscale, earg= .escale)
-        if (residuals) stop("loglikelihood residuals not implemented yet") else {
+        sh = eta2theta(eta[,1], .lshape, earg = .eshape)
+        sc = eta2theta(eta[,2], .lscale, earg = .escale)
+        if (residuals) stop("loglikelihood residuals not ",
+                            "implemented yet") else {
             sum(w * dbisa(x=y, shape=sh, scale=sc, log = TRUE))
         }
-    } , list( .lshape=lshape, .lscale=lscale,
-              .eshape=eshape, .escale=escale ))),
+    } , list( .lshape = lshape, .lscale = lscale,
+              .eshape = eshape, .escale = escale ))),
     vfamily=c("bisa"),
     deriv=eval(substitute(expression({
-        sh = eta2theta(eta[,1], .lshape, earg= .eshape)
-        sc = eta2theta(eta[,2], .lscale, earg= .escale)
+        sh = eta2theta(eta[,1], .lshape, earg = .eshape)
+        sc = eta2theta(eta[,2], .lscale, earg = .escale)
         dl.dsh = ((y/sc - 2 + sc/y) / sh^2 - 1) / sh 
-        dl.dsc = -0.5 / sc + 1/(y+sc) + sqrt(y) * ((y+sc)/y) * 
+        dl.dsc = -0.5 / sc + 1/(y+sc) + sqrt(y) * ((y+sc)/y) *
                  (sqrt(y/sc) - sqrt(sc/y)) / (2 * sh^2 * sc^1.5)
-        dsh.deta = dtheta.deta(sh, .lshape, earg= .eshape)
-        dsc.deta = dtheta.deta(sc, .lscale, earg= .escale)
+        dsh.deta = dtheta.deta(sh, .lshape, earg = .eshape)
+        dsc.deta = dtheta.deta(sc, .lscale, earg = .escale)
         w * cbind(dl.dsh * dsh.deta, dl.dsc * dsc.deta)
-    }) , list( .lshape=lshape, .lscale=lscale,
-               .eshape=eshape, .escale=escale ))),
+    }) , list( .lshape = lshape, .lscale = lscale,
+               .eshape = eshape, .escale = escale ))),
     weight=eval(substitute(expression({
         wz = matrix(as.numeric(NA), n, M)  # Diagonal!!
         wz[,iam(1,1,M)] = 2 * dsh.deta^2 / sh^2
         hfunction = function(alpha)
-            alpha * sqrt(pi/2) - pi * exp(2/alpha^2) * (1-pnorm(2/alpha))
+            alpha * sqrt(pi/2) - pi * exp(2/alpha^2) *
+                                 pnorm(2/alpha, lower.tail = FALSE)
         wz[,iam(2,2,M)] = dsc.deta^2 * (sh * hfunction(sh)  / sqrt(2*pi) +
                           1) / (sh*sc)^2
         w * wz
-    }), list( .zero=zero ))))
+    }), list( .zero = zero ))))
 }
 
 
